/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/
/*
    mxcanbus_win.c

    Routines to operate canbus ports 

    2009-07-25	CF Lin
		new release
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "mxcanbus.h" /* prototype */

int
mxcan_get_parameters( unsigned int fd, CANPRM *param) 
{
	return ioctl(fd, IOCTL_GET_PARAMETERS, param);
}

int
mxcan_set_parameters( unsigned int fd, CANPRM *param) 
{
	return ioctl(fd, IOCTL_SET_PARAMETERS, param);
}

int	
mxcan_set_bus_timing ( unsigned int fd, unsigned int speed)
{
	CANPRM param;

	if (mxcan_get_parameters(fd, &param))
		return -1;
	param.bus_timing = speed;
	if (mxcan_set_parameters(fd, &param))
		return -2;
	else
		return 0;
}

/* Note: this function return 0 on failure, otherwise returning timming */
unsigned int	
mxcan_get_bus_timing(unsigned int fd)
{
	CANPRM param;

	if (mxcan_get_parameters(fd, &param))
		return 0;
	else
		return param.bus_timing;
}

int 
mxcan_purge_buffer( unsigned int fd, unsigned int purge)
{

	if (purge==1)
		return (ioctl( fd, IOCTL_PURGE_RX_BUFFER) != 0)? -1:0;
	else if (purge==2)
		return (ioctl( fd, IOCTL_PURGE_TX_BUFFER) != 0)? -2:0;
	else 
		if (ioctl( fd, IOCTL_PURGE_RX_BUFFER) != 0) return -3;
		if (ioctl( fd, IOCTL_PURGE_TX_BUFFER) != 0) return -4;
	return 0;
}

int
mxcan_set_nonblocking(unsigned int fd)
{
	int ret;
	long flag = fcntl(fd, F_GETFL, 0);
	return fcntl(fd, F_SETFL, flag | O_NONBLOCK);
}

int	
mxcan_inqueue(unsigned int fd)
{
	unsigned int bytes = 0;
	int ret;
	ret = ioctl(fd, IOCTL_GET_RX_BYTES, &bytes);
	printf("inqueue ret=%d\n", ret);
	if (ret < 0) 
		return ret;
	else 
		return bytes;
}


int
mxcan_outqueue( unsigned int fd )
{
	int bytes = 0;
	int ret;

	ret = ioctl(fd, IOCTL_GET_TX_BYTES, &bytes);
	if (ret < 0) 
		return ret;
	else 
		return bytes;
}



int	
mxcan_get_stat(unsigned int fd, CANBST *stat)
{
	return ioctl(fd, IOCTL_GET_DEVICE_STATISTIC, stat); 
}

int	
mxcan_get_registers ( unsigned int fd, unsigned char *buffer, int num)
{
	return ioctl(fd, IOCTL_GET_DEVICE_REGISTERS, buffer);
}

unsigned int
mxcan_open(int port)
{
	unsigned int fd;
	char szPort[20];
	port --;
 	sprintf((char*) szPort, "/dev/can%d", port);
	fd = open( (const char*) szPort, O_RDWR);
	if (fd == -1) fd = 0; /* return 0 for fail */
	return fd;
}
