/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/

#ifndef _DIGIO_H
#define _DIGIO_H

#if !defined(_WIN32_WCE) && !defined(WIN32)
typedef int HANDLE; /* to have a uniform handler type presentation */
#else
#include <windows.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef EPM3438  // Jared, 08-10-2010, for EPM-3438(begin)

#define BOARDCNT 2 // Jared, manage multiple boards
/*
 * open an access to digit io pins on EPM-3438
	Inputs:
		<brdno> the first or second board
		0 for the first EPM-3438 counter. 1 for the second EPM-3438 counter.
	Returns:
		a file descriptor of the access
 */
HANDLE
mxdgio_epm3438_open(int brdno);

/*
 * Get the counter value
	Inputs:
		<fd> access to the conter device
	Returns:
		the counter value
 */
int mxdgio_epm3438_get_counter(HANDLE fd);

/*
 * Clear the counter value
	Inputs:
		<fd> access to the conter device
	Returns:
		0:clear success; -1: clear fail
 */
int mxdgio_epm3438_clear_counter(HANDLE fd);
#endif // Jared, 08-10-2010, for EPM-3438(end)

/*  open an access to digit io pins
    Returns:
	a file descriptor of the access
*/
HANDLE
mxdgio_open(void);

/*  get the signal state of a digit input port
    Inputs:
	<fd> the access to the device
	<port> port #
    Returns:
	1 or 0 on success, -1 on failure
*/
int
mxdgio_get_input_signal(HANDLE fd, int port);

/*  get the signal state of a digit output port 
    Inputs:
	<fd> the access to the device
	<port> port #
    Returns:
	1 or 0 on success, -1 on failure
*/
int	
mxdgio_get_output_signal(HANDLE fd, int port);

/*  pass a high signal to a digit output port
    Inputs:
	<fd> the access to the device
	<port> port #
    Returns:
	0 on success, -1 on failure
*/
int	
mxdgio_set_output_signal_high(HANDLE fd, int port);

/*  pass a low signal to a digit output port
    Inputs:
	<fd> the access to the device
	<port> port #
    Returns:
	0 on success, -1 on failure
*/
int	
mxdgio_set_output_signal_low(HANDLE fd, int port);

/* close the access to the device */
void
mxdgio_close(HANDLE fd);

#ifdef __cplusplus
}
#endif

#endif

